<?php
/**
 * Joomla! 1.5 component JOaktree
 * file		jt_admin model - jt_admin.php
 *
 * @version	1.0
 * @author	Niels van Dantzig
 * @package	Joomla
 * @subpackage	JOaktree
 * @license	GNU/GPL
 *
 * Viewer for genealogical data for Joomla!
 *
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

// Import Joomla! libraries
jimport('joomla.application.component.model');

class JoaktreeModelJt_admin extends JModel {

	var $_persons;
	var $_trees;
	var $_pagination = null;
	var $_total         = null;

	function __construct() {
		parent::__construct();

		global $mainframe, $option;		
		$context	= 'com_joaktree.jt_admin.list.';
		
		// Get the pagination request variables
		$limit		= $mainframe->getUserStateFromRequest( 'global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int' );
		$limitstart	= $mainframe->getUserStateFromRequest( $context.'limitstart',	'limitstart',	0, 'int' );
		
		// In case limit has been changed, adjust limitstart accordingly
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);

	}

	function _buildQuery() {
		// Get the WHERE and ORDER BY clauses for the query
		$where   =  $this->_buildContentWhere();
		$orderby =  $this->_buildContentOrderBy();
		
		$query = 'SELECT '
                        .'  jpn.id                	AS id '
			.', jan.default_tree_id   	AS default_tree_id '
			.', jpn.firstName         	AS firstName '
			.', jpn.patronym         	AS patronym '
			.', CONCAT_WS('.$this->_db->Quote(', ').',jpn.familyName,jpn.namePreposition)	AS familyName '
			.', jpn.sex               	AS sex '
			.', jpn.birthDate         	AS birthDate '
			.', jpn.birthPlace        	AS birthPlace '
			.', jpn.deathDate         	AS deathDate '
			.', jpn.deathPlace        	AS deathPlace '
			.', jpn.burialDate        	AS burialDate '
			.', jpn.burialPlace       	AS burialPlace '
			.', CONCAT_WS( ' . $this->_db->Quote(' - ')
			.'	, IFNULL(SUBSTR( jpn.birthDate, -4 ), '.$this->_db->Quote('?').' ) '
			.'	, SUBSTR(IFNULL( jpn.deathDate, jpn.burialDate ), -4) ) '
			.'   				AS period '
			.', jan.published         	AS published '
			.', jan.living            	AS living '
			.', jan.page              	AS page '
			.', jte.name              	AS familyTree ' 
			.'FROM  (    #__joaktree_persons jpn '
			.'      JOIN #__joaktree_admin_persons jan '
			.'      ON ( jan.id = jpn.id ) '
			.'      ) '
                        .'LEFT JOIN #__joaktree_trees jte '
                        .'ON ( jte.id = jan.default_tree_id ) '
                        . $where
                        . $orderby;
			
		return $query;
	}

	function _buildContentWhere() {
		global $mainframe;
		
		$context		= 'com_joaktree.jt_admin.list.';
		$filter_state		= $mainframe->getUserStateFromRequest( $context.'filter_state',		'filter_state',		'',	'word' );
		$filter_living		= $mainframe->getUserStateFromRequest( $context.'filter_living',	'filter_living',	'',	'word' );
		$filter_page		= $mainframe->getUserStateFromRequest( $context.'filter_page',		'filter_page',		'',	'word' );
		$filter_tree		= $mainframe->getUserStateFromRequest( $context.'filter_tree',		'filter_tree',		0,	'int' );
		$search1		= $mainframe->getUserStateFromRequest( $context.'search1',		'search1',		'',	'string' );
		$search1		= JString::strtolower( $search1 );
		$search2		= $mainframe->getUserStateFromRequest( $context.'search2',		'search2',		'',	'string' );
		$search2		= JString::strtolower( $search2 );
		$search3		= $mainframe->getUserStateFromRequest( $context.'search3',		'search3',		'',	'string' );
		$search3		= JString::strtolower( $search3 );
		
		$where = array();
		
		if ($search1) {
			$where[] = 'LOWER(jpn.firstName) LIKE '.$this->_db->Quote('%'.$search1.'%');
		}
		
		if ($search2) {
			$where[] = 'LOWER(jpn.patronym) LIKE '.$this->_db->Quote('%'.$search2.'%');
		}
		
		if ($search3) {
			$where[] = 'LOWER(jpn.familyName) LIKE '.$this->_db->Quote('%'.$search3.'%');
		}
		
		if ( $filter_state ) {
			if ( $filter_state == 'P' ) {
				$where[] = 'jan.published = 1';
			} else if ($filter_state == 'U' ) {
				$where[] = 'jan.published = 0';
			}
		}
		
		if ( $filter_living ) {
			if ( $filter_living == 'L' ) {
				$where[] = 'jan.living = 1';
			} else if ($filter_living == 'D' ) {
				$where[] = 'jan.living = 0';
			}
		}
		
		if ( $filter_page ) {
			if ( $filter_page == 'Y' ) {
				$where[] = 'jan.page = 1';
			} else if ($filter_page == 'N' ) {
				$where[] = 'jan.page = 0';
			}
		}
		
		if ( $filter_tree != 0 ) {
			$where[] = 'jan.default_tree_id = ' . $filter_tree;
		}
		
		$where 		= ( count( $where ) ? ' WHERE '. implode( ' AND ', $where ) : '' );
		
		return $where;
	}

	function _buildContentOrderBy() {
		global $mainframe;
		
		$context		= 'com_joaktree.jt_admin.list.';
		$filter_order		= $mainframe->getUserStateFromRequest( $context.'filter_order',		'filter_order',		'jpn.id',	'cmd' );
		$filter_order_Dir	= $mainframe->getUserStateFromRequest( $context.'filter_order_Dir',	'filter_order_Dir',	'',				'word' );
		
		if ($filter_order == 'jpn.id'){
			$orderby 	= ' ORDER BY jpn.id '.$filter_order_Dir;
		} else {
			$orderby 	= ' ORDER BY '.$filter_order.' '.$filter_order_Dir.'  ';
		}
		
		return $orderby;
	}

	function getPersons() {
		$query = $this->_buildQuery();
		$this->_persons = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit'));
		
		return $this->_persons;
	}

   	function getTrees() {
		$query = 'SELECT id, name FROM #__joaktree_trees ORDER BY name';
		$this->_trees = $this->_getList( $query );
		
		return $this->_trees;
	}

	function publish() {
		$cids = JRequest::getVar( 'cid', array(), 'post', 'array' );
		
		foreach ($cids as $cid_num => $cid) {   
			$query = 'UPDATE #__joaktree_admin_persons '
				.'SET published = !published '
				.'WHERE id = ' . $this->_db->quote( $cid );
			$msg = $this->_getListCount( $query );
		}
		
		return count($cids) . ' person(s) updated';
	}

	function publishAll() {
		$cids = JRequest::getVar( 'cid', array(), 'post', 'array' );
		
		foreach ($cids as $cid_num => $cid) {   
			$query = 'UPDATE #__joaktree_admin_persons '
				.'SET published = 1 '
				.'WHERE id = ' . $this->_db->quote( $cid );
			$msg = $this->_getListCount( $query );
		}
		
		return count($cids) . ' person(s) updated';
	}

	function unpublishAll() {
		$cids = JRequest::getVar( 'cid', array(), 'post', 'array' );
		
		foreach ($cids as $cid_num => $cid) {   
			$query = 'UPDATE #__joaktree_admin_persons '
				.'SET published = 0 '
				.'WHERE id = ' . $this->_db->quote( $cid );
			$msg = $this->_getListCount( $query );
		}
		
		return count($cids) . ' person(s) updated';
	}

	function living() {
		$cids = JRequest::getVar( 'cid' );
		
		foreach ($cids as $cid_num => $cid) {   
			$query = 'UPDATE #__joaktree_admin_persons '
				.'SET living = !living '
				.'WHERE id = ' . $this->_db->quote( $cid );
			$msg = $this->_getListCount( $query );
		}
		
		return count($cids) . ' person(s) updated';
	}

	function livingAll() {
		$cids = JRequest::getVar( 'cid' );
		
		foreach ($cids as $cid_num => $cid) {   
			$query = 'UPDATE #__joaktree_admin_persons '
				.'SET living = 1 '
				.'WHERE id = ' . $this->_db->quote( $cid );
			$msg = $this->_getListCount( $query );
		}
		
		return count($cids) . ' person(s) updated';
	}

	function notLivingAll() {
		$cids = JRequest::getVar( 'cid' );
		
		foreach ($cids as $cid_num => $cid) {   
			$query = 'UPDATE #__joaktree_admin_persons '
				.'SET living = 0 '
				.'WHERE id = ' . $this->_db->quote( $cid );
			$msg = $this->_getListCount( $query );
		}
		
		return count($cids) . ' person(s) updated';
	}

	function page() {
		$cids = JRequest::getVar( 'cid' );
		
		foreach ($cids as $cid_num => $cid) {   
			$query = 'UPDATE #__joaktree_admin_persons '
				.'SET page = !page '
				.'WHERE id = ' . $this->_db->quote( $cid );
			$msg = $this->_getListCount( $query );
		}
		
		return count($cids) . ' person(s) updated';
	}

	function pageAll() {
		$cids = JRequest::getVar( 'cid' );
		
		foreach ($cids as $cid_num => $cid) {   
			$query = 'UPDATE #__joaktree_admin_persons '
				.'SET page = 1 '
				.'WHERE id = ' . $this->_db->quote( $cid );
			$msg = $this->_getListCount( $query );
		}
		
		return count($cids) . ' person(s) updated';
	}

	function noPageAll() {
		$cids = JRequest::getVar( 'cid' );
		
		foreach ($cids as $cid_num => $cid) {   
			$query = 'UPDATE #__joaktree_admin_persons '
				.'SET page = 0 '
				.'WHERE id = ' . $this->_db->quote( $cid );
			$msg = $this->_getListCount( $query );
		}
		
		return count($cids) . ' person(s) updated';
	}

	function getTotal() {
		// Lets load the content if it doesn't already exist
		if (empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
		}
		
		return $this->_total;
	}

	function getPagination() {
		// Lets load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
		}
		
		return $this->_pagination;
	}
}
?>