<?php
/**
 * Joomla! 1.5 component JOaktree
 * file		jt_familytree model - jt_familytree.php
 *
 * @version	1.0
 * @author	Niels van Dantzig
 * @package	Joomla
 * @subpackage	JOaktree
 * @license	GNU/GPL
 *
 * Viewer for genealogical data for Joomla!
 *
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

JTable::addIncludePath(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_joaktree'.DS.'tables');
JModel::addIncludePath(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_joaktree'.DS.'models');


// Import Joomla! libraries
jimport('joomla.application.component.model');

class JoaktreeModelJt_familytree extends JModel {

	var $_data;
	var $_pagination = null;
	var $_total         = null;

	function __construct() {
		parent::__construct();		


		global $mainframe;		
		$context			= 'com_joaktree.jt_familytree.list.';
		// Get the pagination request variables
		$limit		= $mainframe->getUserStateFromRequest( 'global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int' );

		$limitstart	= $mainframe->getUserStateFromRequest( $context.'limitstart',	'limitstart',	0, 'int' );
		// In case limit has been changed, adjust limitstart accordingly
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);
	}

	function _buildQuery() {
		// Get the WHERE and ORDER BY clauses for the query
		$where      =  $this->_buildContentWhere();
		$orderby	=  $this->_buildContentOrderBy();
		
		$query = 'SELECT jte.*, '
			.'       jpn.firstName, '
			.'       CONCAT_WS('.$this->_db->Quote(' ').',jpn.namePreposition,jpn.familyName) AS familyName '
			.'FROM      #__joaktree_trees jte ' 
			.'LEFT JOIN #__joaktree_persons jpn '
			.'ON (jpn.id = jte.root_person_id)'
			. $where . $orderby;
			
		return $query;
	}

	function _buildContentWhere() {
		global $mainframe;
		
		$context			= 'com_joaktree.jt_familytree.list.';
		$filter_state		= $mainframe->getUserStateFromRequest( $context.'filter_state',		'filter_state',		'',				'word' );
		$search			= $mainframe->getUserStateFromRequest( $context.'search',			'search',			'',				'string' );
		$search			= JString::strtolower( $search );
		
		$where = array();
		
		if ($search) {
			$where[] = 'LOWER(jte.name) LIKE '.$this->_db->Quote('%'.$search.'%');
		}
		
		if ( $filter_state ) {
			if ( $filter_state == 'P' ) {
				$where[] = 'jte.published = 1';
			} else if ($filter_state == 'U' ) {
				$where[] = 'jte.published = 0';
			}
		}
		
		$where 		= ( count( $where ) ? ' WHERE '. implode( ' AND ', $where ) : '' );
		
		return $where;
	}

	function _buildContentOrderBy() {
		global $mainframe;
		
		$context			= 'com_joaktree.jt_familytree.list.';
		$filter_order		= $mainframe->getUserStateFromRequest( $context.'filter_order',		'filter_order',		'jte.id',	'cmd' );
		$filter_order_Dir	        = $mainframe->getUserStateFromRequest( $context.'filter_order_Dir',	'filter_order_Dir',	'',				'word' );
		
		if ($filter_order){
			$orderby 	= ' ORDER BY '.$filter_order.' '.$filter_order_Dir.' ';
		} else {
                        $orderby 	= ' ORDER BY jte.id '.$filter_order_Dir.' ';
		}
		
		return $orderby;
	}

	function getData() {
		// Lets load the content if it doesn't already exist
		if (empty($this->_data))
		{
                     $query = $this->_buildQuery();
                     $this->_data = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit'));
		}
		
		return $this->_data;
	}

	function getTotal() {
		// Lets load the content if it doesn't already exist
		if (empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
		}
		
		return $this->_total;
	}

	function getPagination() {
		// Lets load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
		}
		
		return $this->_pagination;
	}

	function publish() {
		$cids = JRequest::getVar( 'cid', array(), 'post', 'array' );
		
		foreach ($cids as $cid_num => $cid) {   
			$query = 'UPDATE #__joaktree_trees '
				.'SET published = !published '
				.'WHERE id = ' . $cid;
			$msg = $this->_getListCount( $query );
		}
		
		return 'family trees updated';
	}

	function assignFamilyTree() {
		$params		= JComponentHelper::getParams('com_joaktree') ;
		$numTrees	= (int) $params->get('numberTrees', 0);
		$singleTreeId	= (int) $params->get('singleTreeId', 0);

		// Initialize table
		$tree_persons   =& JTable::getInstance('joaktree_tree_persons', 'Table');    
		$tree_persons->truncate();
		
		// empty the default tree attribute for all persons in the system
		$query = 'UPDATE #__joaktree_admin_persons '
			.'SET default_tree_id = null '; 
		$msg = $this->_getListCount( $query );
		
		// Initialize arrays for processing
		$person_now = array();
		$person_sex_now = array();
		$type_now = array();
		$lineage_now = array();
		$person_next = array();
		$person_sex_next = array();
		$type_next = array();
		$lineage_next = array();
		
		// select all published trees
		$query = "SELECT * FROM #__joaktree_trees WHERE published = true ";
		$trees = $this->_getList( $query);
		
		// loop through all published trees
		foreach ($trees as $tree_num => $tree) {   
			// For this tree - initialize some values
			// Current tree
			if ($numTrees == 0) {
				// single tree
				$tree_now = $singleTreeId;
			} else {
				// multiple trees
				$tree_now = $tree->id;
			}
			
			// First person we start with is the root-person of the tree
			// For logic this person is always male (even if she is not)
			$person_now[]  = $tree->root_person_id;
			$person_sex_now[]  = 'M';
			// Therefore the type of this first person (root-person) is R
			$type_now[]    = 'R';
			// lineage is filled with the first person.
			$lineage_now[] = $tree->root_person_id;
			
			// for the next while loop we need some stoppers
			$finished          = false;
			$while_loop_teller = 0;
			
			// In this while loop we are going to walk through the tree 
			// starting with the first person, looking for spouses and children
			// For every found person we will look again (that is the while loop) for relations
			while ($finished == false) {
				// To keep track we count the number of loops
				$while_loop_teller++;
				
				// there is an array of persons to process NOW.
				// loop through every record in this array (i.e. person_now) 
				for ($i=0, $n=count($person_now); $i < $n; $i++)   {
					
					// check that the person-tree combination (primary key) does not exists yet
					if ( ($tree_persons->load( $person_now[$i] . '+' . $tree_now ) == false ) ) {
						
						// the person-tree combination does not exists
						// therefore the attributes for a new record are filled
						// primary key is combination of person-id and tree-id separated by +
						$tree_persons->set( 'id' , $person_now[$i] . '+' . $tree_now ); 
						$tree_persons->set( 'tree_id' , $tree_now ); 
						$tree_persons->set( 'person_id' , $person_now[$i] ); 
						$tree_persons->set( 'type' , $type_now[$i] ); 
						$tree_persons->set( 'lineage' , $lineage_now[$i] );
						
						// insert the new record
						$ret = $tree_persons->_db->insertObject( $tree_persons->_tbl, $tree_persons, $tree_persons->_tbl_key );

						// after the new record is inserted, look for all relations of the newly inserted person
						// the relations are husband or wife (type = P for partner of)
						// and relations are father or mother (type = C for child)
						$query = 'SELECT jrn.to_person_id AS pid '
							.', ' . $this->_db->quote( 'P' ) . ' AS type '
							.',      jpn.sex AS sex '
							.'FROM #__joaktree_relations jrn '
							.'JOIN #__joaktree_persons   jpn '
							.'ON   (jpn.id = jrn.to_person_id) '
							.'WHERE jrn.person_id = ' . $this->_db->quote( $person_now[$i] ) . ' '
							.'AND jrn.type IN ( ' . $this->_db->quote( 'husband' ) . ', ' .  $this->_db->quote( 'wife' ) . ') '
							.'UNION '
							.'SELECT jrn.person_id AS pid '
							.', ' . $this->_db->quote( 'C' ) . ' AS type '
							.',      jpn.sex AS sex '
							.'FROM #__joaktree_relations jrn '
							.'JOIN #__joaktree_persons   jpn '
							.'ON   (jpn.id = jrn.person_id) '
							.'WHERE jrn.to_person_id = ' . $this->_db->quote( $person_now[$i] ) . ' '
							.'AND jrn.type IN ( ' . $this->_db->quote( 'father' ) . ', ' .  $this->_db->quote( 'mother' ) . ') ';
							
						$relations = $this->_getList( $query);
						
						// the relations are selected and stored in an array which 
						// will be processed in the next round of this while-loop
						// therefore the array ends with NEXT
						foreach ($relations as $rel_num => $relation) {    
							// newly retrieved relation (id, type) for process next round
							$person_next[]     = $relation->pid;
							$person_sex_next[] = $relation->sex;
							$type_next[]       = $relation->type;
							
							// if type is P (=partner) this person does not descends
							// from root-persoon. Therefore lineage for this partner (for this tree)
							// is empty. For children the lineage is extended
							// For children the default tree is updated too, if the person is male
							if ( $relation->type == 'P' ) {
								$lineage_next[] = '' ;
							} else {
								$lineage_next[] = $lineage_now[$i] . ' ' . $relation->pid  ;
								
								if ($person_sex_now[$i] == 'M') {
									$query = 'UPDATE #__joaktree_admin_persons '
										.'SET default_tree_id = ' . $tree_now . ' '
										.'WHERE id = ' . $this->_db->quote( $relation->pid ) . ' '; 
									$msg   = $this->_db->setQuery( $query );
									$msg   = $this->_db->query( );
								}
							}
						} // end of loop through relations
					} // end of if statement - adding new record for person-tree combination
				} // end of for-loop for the array being processed NOW
				
				// the NOW array is processed; a NEXT array is filled
				// because the NOW array is done, it is emptied with the array_splice function
				array_splice($person_now, 0);
				array_splice($person_sex_now, 0);
				array_splice($type_now, 0);
				array_splice($lineage_now, 0);
				
				// the emptied NOW array is filled again with the values of the NEXT array
				$person_now      = $person_next;
				$person_sex_now  = $person_sex_next;
				$type_now        = $type_next;
				$lineage_now     = $lineage_next;
				
				// after copying the NEXT array into the NOW array, 
				// the NEXT array is emptied too in order it can be filled in the next round
				array_splice($person_next, 0);
				array_splice($person_sex_next, 0);
				array_splice($type_next, 0);
				array_splice($lineage_next, 0);
				
				// this is a while loop: it can stop (a) when the person_now is empty
				// (that means: there is nothing to be processed anymore) or
				// when a number of loops is reached (just to be on the safe side)
				if ( (count( $person_now) == 0) or ($while_loop_teller > 100)  ) { $finished = true; }
			} // end of while-loop for one tree
		} // end loop throug all published trees
		
		// PERSONS WITH 0 or 1 TREE
		// after filling the tree_person table, the default tree for every person is determined
		// for which it is not yet filled
		if ($numTrees == 0) {
			// everyone gets the same default tree
			$query = 'UPDATE #__joaktree_admin_persons '
				.'SET default_tree_id = ' . $singleTreeId; 
			$msg   = $this->_db->setQuery( $query );
			$msg   = $this->_db->query( );
		} else {
			// different default trees for different persons
			// select all persons in the system without default tree
			$query = 'SELECT jpn.id AS id '
				.'FROM   #__joaktree_persons       jpn '
				.'JOIN   #__joaktree_admin_persons jan '
				.'ON     (jan.id = jpn.id) '
				.'WHERE  jan.default_tree_id IS NULL ';
			$person_ids  = $this->_getList( $query);
			
			// loop through all persons in the system and determine their default tree
			foreach ($person_ids as $pid_num => $pid) {   
				// dummy value - meaning: default tree cannot be determined
				$def_tree_id = null;
				
				// select the candidate trees from the tree_person table
				$query = 'SELECT tree_id '
					.'FROM #__joaktree_tree_persons '
					.'WHERE person_id = ' .  $this->_db->quote( $pid->id ) ;
				$numberTrees = $this->_getListCount( $query );
				
				// if 0 trees are found, there is no tree (and thus no default tree) for this person
				if ($numberTrees == 0) { $def_tree_id = 0; }
				
				// if 1 tree is found, this tree is the default tree for this person
				if ($numberTrees == 1) { 
					$def_tree_id = $this->_db->loadResult( $query );
				}
				
				$query = 'UPDATE #__joaktree_admin_persons '
					.'SET default_tree_id = ' . $def_tree_id . ' '
					.'WHERE id = ' . $this->_db->quote( $pid->id ) . ' '; 
				$msg   = $this->_db->setQuery( $query );
				$msg   = $this->_db->query( );
			}
			
			// PERSONS WITH MORE THAN 1 TREE
			// after filling the tree_person table, the default tree for every person is determined
			// for which it is not yet filled
			// select all persons in the system without default tree
			$query = 'SELECT jpn.id AS id '
				.'FROM   #__joaktree_persons       jpn '
				.'JOIN   #__joaktree_admin_persons jan '
				.'ON     (jan.id = jpn.id) '
				.'WHERE  jan.default_tree_id IS NULL ';
			$person_ids  = $this->_getList( $query);
			
			// loop through all persons in the system and determine their default tree
			foreach ($person_ids as $pid_num => $pid) {   
				// dummy value - meaning: default tree cannot be determined
				$def_tree_id = null;
				
				// select the default tree from the first father
				$query = 'SELECT    jan.default_tree_id AS tree_id '
					.'FROM      #__joaktree_relations     jrn '
					.'JOIN      #__joaktree_admin_persons jan '
					.'ON        (jan.id = jrn.to_person_id) '
					.'WHERE     jrn.person_id = ' .  $this->_db->quote( $pid->id )
					.'AND       jrn.type = ' . $this->_db->quote( 'father' )
					.'ORDER BY  jrn.orderNr ';
				
				$numberRecords = $this->_getListCount( $query );
				
				if ($numberRecords > 0) {
					$def_tree_id = $this->_db->loadResult( $query );
				}
				
				// if father has no default tree, look for the default tree of the mother
				if ( ($def_tree_id == 0) or ($def_tree_id == null) ) { 
					// select the default tree from the first mother
					$query = 'SELECT    jan.default_tree_id AS tree_id '
						.'FROM      #__joaktree_relations     jrn '
						.'JOIN      #__joaktree_admin_persons jan '
						.'ON        (jan.id = jrn.to_person_id) '
						.'WHERE     jrn.person_id = ' .  $this->_db->quote( $pid->id )
						.'AND       jrn.type = ' . $this->_db->quote( 'mother' )
						.'ORDER BY  jrn.orderNr ';
					
					$numberRecords = $this->_getListCount( $query );
					
					if ($numberRecords > 0) {
						$def_tree_id = $this->_db->loadResult( $query );
					}
					
					// if mother has no default tree, look for the default tree of the partner
					if ( ($def_tree_id == 0) or ($def_tree_id == null) ) { 
						// select the default tree from the first partner
						$query = 'SELECT    jan.default_tree_id AS tree_id '
							.'FROM      #__joaktree_relations     jrn '
							.'JOIN      #__joaktree_admin_persons jan '
							.'ON        (jan.id = jrn.to_person_id) '
							.'WHERE     jrn.person_id = ' .  $this->_db->quote( $pid->id )
							.'AND       jrn.type IN ( ' . $this->_db->quote( 'husband' ) . ', ' . $this->_db->quote( 'wife' ) . ') '
							.'ORDER BY  jrn.orderNr ';
						
						$numberRecords = $this->_getListCount( $query );
						
						if ($numberRecords > 0) {
							$def_tree_id = $this->_db->loadResult( $query );
						}
					} // end if - mother check
				} // end if  - father check
				
				$query = 'UPDATE #__joaktree_admin_persons '
					.'SET default_tree_id = ' . $def_tree_id . ' '
					.'WHERE id = ' . $this->_db->quote( $pid->id ) . ' '; 
				$msg   = $this->_db->setQuery( $query );
				$msg   = $this->_db->query( );
			}
		}
		
		return 'Assign family trees is finished' ;
	}
}
?>