<?php
/**
 * Joomla! 1.5 component JOaktree
 * file		jt_gedcomfile model - jt_gedcomfile.php
 *
 * @version	1.0.1
 * @author	Niels van Dantzig
 * @package	Joomla
 * @subpackage	JOaktree
 * @license	GNU/GPL
 *
 * Viewer for genealogical data for Joomla!
 *
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

// Import Joomla! libraries
jimport( 'joomla.filesystem.file' );

JTable::addIncludePath(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_joaktree'.DS.'tables');


class Joaktree_gedcomfile extends JFile {

	private $_db;

	function __construct() {
		// parent::__construct();
		$this->_db              = & JFactory::getDBO(); 
		$this->params           = JComponentHelper::getParams('com_joaktree') ;
		$this->show_until_year  = (int) $this->params->get('show_until_year', 2100);
		$this->show_living_year = (int) $this->params->get('show_living_year', 1800);
		$this->age_no_page      = (int) $this->params->get('age_no_page', 0);
		
		$truncate_rel_value	= (int) $this->params->get('truncrelations');
		if ($truncate_rel_value == 1) {
			$this->trunc_rel = true;
		} else {
			$this->trunc_rel = false;			
		}
		
		$this->patronymSetting	= (int) $this->params->get('patronym');
		$this->patronymString	= $this->params->get('patronymSeparation');
		$this->familynameSetting = (int) $this->params->get('familyname');
	}

	/*
	** Function to set the "show" attribute automatically based on
	** information about the person.
	*/
	private function admin_show_person () {
		$show = false;
		
		// if person has no death or burial information, she/he still may be living
		if (     ($this->persons->get('deathDate')   == null )
		     and ($this->persons->get('deathPlace')  == null )
		     and ($this->persons->get('burialDate')  == null )
		     and ($this->persons->get('burialPlace') == null )
		   ) 
		{
			// person still may be living. Therefore check the brith year
			$birthYear = (int) substr($this->persons->get('birthDate'), -4);

			// if birhtyear is (a) present and (b) before the cut of year
			// (that is parameter "show until year") the person will be shown 
			if (     ($birthYear != null )
			     and ($birthYear >   1000 ) 
			     and ($birthYear <= $this->show_until_year )
			   ) 
			{
				// persons will be shown
				$show = true;
			} else {
				// persons will NOT be shown
				$show = false;
			}
		} else {
			// persons is dead and will be shown
			$show = true;
		}
		
		return $show;
	}


	/*
	** Function to set the "living" attribute automatically based on
	** information about the person.
	*/
	private function admin_living_person () {
		$living = true;
		
		// if person has no death or burial information, she/he still may be living
		if (     ($this->persons->get('deathDate')   == null )
		     and ($this->persons->get('deathPlace')  == null )
		     and ($this->persons->get('burialDate')  == null )
		     and ($this->persons->get('burialPlace') == null )
		   ) 
		{
			// person still may be living. Therefore check the brith year
			$birthYear = (int) substr($this->persons->get('birthDate'), -4);
			
			// if birhtyear is (a) present and (b) before the cut of year
			// (that is parameter "show living year") the is living 
			if (     ($birthYear != null )
			     and ($birthYear >   1000 ) 
			     and ($birthYear <= $this->show_living_year )
			   ) 
			{
				// persons does not live anymore
				$living = false;
			} else {
				// persons still lives
				$living = true;
			}
		} else {
			// persons does not live anymore
			$living = false;
		}
		
		return $living;
	}


	/*
	** Function to set the "link" for a person to his/her own page
	** Children who died before a certain age (that is parameter "age no page"), 
	** do not have their own page
	*/
	private function admin_page_person () {
		$show_page = true;
		
		// check the year of death
		$deathYear = (int) substr($this->persons->get('deathDate'), -4);
		
		// if no year of death is found, check the year of burial
		// year of burial is the same as year of death
		if ( ($deathYear == null) or ($deathYear == 0) ) {
			$deathYear = (int) substr($this->persons->get('burialDate'), -4);
		}
		
		if ( ($deathYear == null) or ($deathYear == 0) ) {
			// year of death is not known. age of person cannot be established.
			// person will have its own page.
			$show_page = true;
		} else {
			// check the year of birth
			$birthYear = (int) substr($this->persons->get('birthDate'), -4);
			
			if ( ($birthYear == null) or ($birthYear == 0) ) {
				// year of death is known, but year of birth is not known.
				// age of person cannot be established.
				// person will have its own page.
				$show_page = true;
			} else {
				// year of death and year of birth are known.
				if ( ($deathYear - $birthYear) <= $this->age_no_page ) {
					// person died before the cut off age
					// person will NOT have its own page
					$show_page = false;
				} else {
					// person died after the cut off age
					// person will have its own page
					$show_page = true;
				}
			}
		}
		
		return $show_page;
	}


	/*
	** Function to set administration setting automatically for newly added persons
	** Settings can later be modified by administrator.
	*/
	private function admin_person() {
		$admin = & JTable::getInstance('joaktree_admin_persons', 'Table');
		
		// check whether person exists
		$admin->set('id'       , $this->persons->get('id')     );
		$exists = $admin->person_exists();
		
		if ($exists) {
			// person exists: do nothing do not update admin table
			return true;
		} else {
			// insert person into admin table
			$admin->set('published', $this->admin_show_person()    );
			$admin->set('living'   , $this->admin_living_person () );
			$admin->set('page'     , $this->admin_page_person ()   );
		
			$ret = $admin->_db->insertObject( $admin->_tbl, $admin, $admin->_tbl_key );
		
			return $ret;
		}
	}


	/*
	** Function to set initial source info
	*/
	private function setSource( $source_id ) {
		// if a previous source exist, this should first be processed
		if ( $this->person_sources->get('source_id') ) {
			$ret = $this->add_person_source();
		}
														
		// fill the source information. source text may follow.
		// source is not processed yet.
		$person_id = $this->person_sources->get('person_id');
		$this->person_sources->set('source_id' , $source_id );
		$this->person_sources->set('id'        , $person_id . $source_id );
		$this->person_sources->set('sourceText', ' ' );
		
		return true;
	}


	/*
	** Function to retrieve parent information from gedcom tables based
	** on person and family ID.
	*/
	private function parents( $person_id, $family_id ) {
		// initialize values
		$this->relations->loadEmpty();
		$this->relations->set('person_id', $person_id);
		$father_id = null;
		$mother_id = null;
		$number_child = 0;
		
		// set up query for selecting family information and execute query
		$query = 'SELECT   * '
		        .'FROM     #__joaktree_gedcom_objectlines '
			.'WHERE    object_id = ' . $this->_db->Quote( $family_id ). ' '
			.'ORDER BY order_nr';
		
		$this->_db->setQuery( $query );
		$fam_lines = $this->_db->loadAssocList();
		
		// loop through the resulting set of family object lines and extract information
		foreach ($fam_lines as $fam_line_num => $fam_line) {
			// check the level. only level 1 is of interest.
			switch ($fam_line['level']) {
				// check the tag for level 1. only HUSB, WIFE and CHIL are of interest
				case "1": 	$information = $fam_line['tag'];
							switch ($information) {
								case "HUSB":	// the husband of this family is the father of the person
												$father_id = ltrim(rtrim($fam_line['value'], '@'), '@');
												BREAK;
								case "WIFE":	// the wife of this family is the mother of the person
												$mother_id = ltrim(rtrim($fam_line['value'], '@'), '@');
												BREAK;
								case "CHIL": 	// to save the order number of the person - who is a 
												// child in this family - we compare the id of every child
												// with the id of the person, while counting the number of childs
												// when the id's are equal, the count is the order number
												if ( $person_id == ltrim(rtrim($fam_line['value'], '@'), '@') ) {
													$number_child++;
													$this->relations->set('orderNr', $number_child);
												} else {
													$number_child++;
												} 
												BREAK;
								default: 		BREAK;
							}
							BREAK;
				default: 	BREAK;
			}
		} // end of loop
		
		// if we found a father, information is written to relation table
		$ret = true;
		if ( $father_id !== null ) {
			$this->relations->set('type'        , 'father');
			$this->relations->set('to_person_id', $father_id);
			$this->relations->set('id'          , $person_id . $father_id);
			
			// $ret = $this->relations->_db->insertObject( $this->relations->_tbl, $this->relations, $this->relations->_tbl_key ); 
			$ret = $this->relations->store(); 
		}
		
		// if we found a mother, information is written to relation table
		if ( ($ret) and ($mother_id !== null) ) {
			$this->relations->set('type'        , 'mother');
			$this->relations->set('to_person_id', $mother_id);
			$this->relations->set('id'          , $person_id . $mother_id);
			
			// $ret = $this->relations->_db->insertObject( $this->relations->_tbl, $this->relations, $this->relations->_tbl_key ); 
			$ret = $this->relations->store(); 
		}
                     
		return $ret;
	}


	/*
	** Function to find for every person in table his/her patronym from relation table.
	*/
	private function getPatronyms() {
		// set up query for selecting person_ids and father's first name
		$query = 'SELECT  jpn.id '
			.',	  jpn2.firstName	AS patronym '
			.'FROM '
			.'(		#__joaktree_persons   jpn '
			.'LEFT JOIN	#__joaktree_relations jrn '
			.'ON		( jrn.person_id 	= jpn.id '
			.'		AND jrn.type 		= ' .  $this->_db->Quote( 'father' ) . ' '
			.'		AND jrn.to_person_id    = '
			.'( SELECT	MIN( jrn2.to_person_id ) '
			.'  FROM	#__joaktree_relations jrn2 '
			.'  WHERE	jrn2.person_id = jrn.person_id '
			.'  AND		jrn2.type = '.  $this->_db->Quote( 'father' ) . ' '
			.') ) ) '
			.'LEFT JOIN	#__joaktree_persons	jpn2 '
			.'ON		( jpn2.id = jrn.to_person_id ) ';
			
		$this->_db->setQuery( $query );
		$persons = $this->_db->loadAssocList();
		
		// loop through the resulting set of family object lines and extract information
		foreach ($persons as $person_num => $person) {
			if ($person[ 'patronym' ]!= null) {
				$query = 'UPDATE #__joaktree_persons '
					.'SET	 patronym = ' . $this->_db->Quote( $person[ 'patronym' ] ) . ' '
					.'WHERE	 id = ' . $this->_db->Quote( $person[ 'id' ] );
					$this->_db->setQuery( $query );
					$ret = $this->_db->query();
			}
 		}
		
		return $ret;
	}


	/*
	** Function to retrieve spouse information from gedcom tables based
	** on person and family ID.
	*/
	private function spouse( $person_id, $family_id) {
		// initialize values
		$this->relations->loadEmpty();
		$this->relations->set('person_id', $person_id);
		$this->number_spouse++;
		$this->relations->set('orderNr', $this->number_spouse);
		
		// set up query for selecting family information and execute query
		$query = 'SELECT   * '
		        .'FROM     #__joaktree_gedcom_objectlines '
			.'WHERE    object_id = ' . $this->_db->Quote( $family_id ) . ' '
			.'ORDER BY order_nr';
			
		$this->_db->setQuery( $query );
		$fam_lines = $this->_db->loadAssocList();
		
		// loop through the resulting set of family object lines and extract information
		foreach ($fam_lines as $fam_line_num => $fam_line) {
			// level swicth
			switch ($fam_line['level']) {
				case "1": // get tag information and save it in $information
					  $information = $fam_line['tag'];
					  
					  // level 1: tag switch
					  switch ($information) {
						  case "HUSB":	// if HUSB is different than person, 
						  		// then he is the spouse
								$to_person_id = ltrim(rtrim($fam_line['value'], '@'), '@');
								if ( $to_person_id !== $person_id ) {
									$this->relations->set('type', 'husband');
									$this->relations->set('id', $person_id . $to_person_id);
									$this->relations->set('to_person_id', $to_person_id);
								}
								BREAK;
						  case "WIFE":	// if WIFE is different than person, 
						  		// then she is the spouse
								$to_person_id = ltrim(rtrim($fam_line['value'], '@'), '@');
								if ( $to_person_id !== $person_id ) {
									$this->relations->set('type', 'wife');
									$this->relations->set('id', $person_id . $to_person_id);
									$this->relations->set('to_person_id', $to_person_id);
								}
								BREAK;
						  case "MARR": 	// $information is set to MARR. this will be used for level 2 
								BREAK;
						  default:	BREAK;
					  }
					  BREAK;
					  
				case "2": // get tag information and execute level 2: tag switch
					  switch ($fam_line['tag']) {
						  case "TYPE":	// type is only used to set the type of marriage
						  		// to religious in case level 1 tag = MARR
								if (    ($information == 'MARR') 
								    and ($fam_line['value'] == 'RELIGIOUS') 
								   ) 
								{
									$information = 'MARR_REL'; 
								}
								BREAK;
						  case "DATE":	// set the date of marriage or religous marriage
								if ($information == 'MARR') { 
									$this->relations->set( 'marriageDate', $fam_line['value']); 
								}
								if ($information == 'MARR_REL') { 
									$this->relations->set( 'rel_marriageDate', $fam_line['value']); 
								}
								BREAK;                      
						  case "PLAC":	// set the place of marriage or religous marriage
								if ($information == 'MARR') { 
									$this->relations->set( 'marriagePlace', $fam_line['value']); 
								}
								if ($information == 'MARR_REL') { 
									$this->relations->set( 'rel_marriagePlace', $fam_line['value']); 
								}
								BREAK;
						  case "SOUR":	// collect all sources for this person
								$ret = $this->setSource( rtrim(ltrim( $row_line['value'], '@'), '@') );
								BREAK;
						  default:	BREAK;
					  }
					  BREAK;
				default:  // get tag information and execute. this it to retrieve possible source info
					  switch ($row_line['tag']) {
						  case "PAGE":	// found source text. save it but do not process it yet.
								$this->person_sources->set('sourceText', $row_line['value'] );
								BREAK;
						  default: 	BREAK;
					  }
					  BREAK;
			} // end level switch
		} // end loop through family object lines
		
		// insert the spousal information in relation table
		$ret = $this->relations->store(); 
		
		return $ret;
	}

	
	/*
	** Function to extract prepositions (dutch) from family name
	*/
	private function setFamilyName($inputName) {
		$nameElement  = explode(" ", $inputName);
		$num_prepos = 0;
		
		if ($this->familynameSetting == 1) {
			// names are separated into two parts
			switch ( strtoupper($nameElement[0]) ) {
				case "IN":	// do nothing
				case "OP":	// do nothing
				case "TOT":	// do nothing
				case "UIJT":	// do nothing
				case "UIT":	// do nothing
				case "UYT":	// do nothing
				case "VAN":	// first part is Dutch preposition - check for second part
						switch ( strtoupper($nameElement[1]) ) {
							case "DE":	// do nothing
							case "DER":	// do nothing
							case "DEN":	// do nothing
							case "HET":	// do nothing
							case "'T":	// do nothing
							case "TE":	// do nothing
							case "TER":	// do nothing
							case "TEN":	// do nothing
							case "EEN":	// second part has a Dutch particle
									$num_prepos = 2;
									BREAK;
							default:	// no particle - just a preposition
									$num_prepos = 1;
									BREAK;
						}
						BREAK;
				// no preposition check for Dutch particles
				case "DE":	// do nothing
				case "DER":	// do nothing
				case "DEN":	// do nothing
				case "HET":	// do nothing
				case "'T":	// do nothing
				case "TE":	// do nothing
				case "TER":	// do nothing
				case "TEN":	// do nothing
				case "EEN":	// second part has a Dutch particle
						$num_prepos = 1;
						BREAK;
				default:	// nothing - just a family name
						$num_prepos = 0;
						BREAK;
			}
			
			if ($num_prepos == 1) {
				// namePreposition exists of 1 element
				$this->persons->set( 'namePreposition', $nameElement[0]);
				
				// remove the one element from the array of name elements
				$tmp = array_shift($nameElement);
				
				// family name is constructed from the remaining elements and saved
				$familyName        = implode(" ", $nameElement);
				$this->persons->set( 'familyName', $familyName);
			} else if ($num_prepos == 2) {
				// namePreposition exists of 2 element
				$this->persons->set( 'namePreposition', $nameElement[0] .' '. $nameElement[1]);
				
				// remove the two elements from the array of name elements
				$tmp = array_shift($nameElement);
				$tmp = array_shift($nameElement);
				
				// family name is constructed from the remaining elements and saved
				$familyName        = implode(" ", $nameElement);
				$this->persons->set( 'familyName', $familyName);
			} else {
				// no preposition - family name is saved unchanged
				$this->persons->set( 'namePreposition', null);
				$this->persons->set( 'familyName', $inputName);
			}
		} else {
			// names are left unchanged
			$this->persons->set( 'namePreposition', null);
			$this->persons->set( 'familyName', $inputName);
		}
			
		return true;
	}


	/*
	** Function to add new source information for a specific person.
	** first the function checks whether unique source information already exists for this person
	** if not, then the record is added.
	*/
	private function add_person_source() {
		$query = 'SELECT 1 '
			.'FROM             ' . $this->person_sources->_tbl . ' '
			.'WHERE id       = ' . $this->person_sources->_db->Quote( $this->person_sources->get( 'id') ) . ' '
			.'AND person_id  = ' . $this->person_sources->_db->Quote( $this->person_sources->get( 'person_id') ) . ' '
			.'AND source_id  = ' . $this->person_sources->_db->Quote( $this->person_sources->get( 'source_id') ) . ' '
			.'AND sourceText = ' . $this->person_sources->_db->Quote( $this->person_sources->get( 'sourceText') ) . ' ';
			
		// execute query and retrieve result
		$this->person_sources->_db->setQuery( $query );
		$result = $this->person_sources->_db->loadResult();
		
		// if query has no result, source information does not exists yet and is added here
		if (!$result) {
			$ret = $this->person_sources->_db->insertObject( $this->person_sources->_tbl, $this->person_sources, $this->person_sources->_tbl_key );
			return $ret;
		} else {
			return $result;
		}
	}


	/*
	** Main function to process all sources from gedcom file.
	*/
	function process_sources() {
		// initialize table
		$gedcom_lines = & JTable::getInstance('joaktree_gedcom_objectlines', 'Table');
		$this->sources = & JTable::getInstance('joaktree_sources', 'Table');
		
		// initialize counters
		$teller = 0;
		
		// select all sources from gedcom table
		$query = 'SELECT * '
			.'FROM   #__joaktree_gedcom_objects '
			.'WHERE  tag = ' . $this->_db->Quote( 'SOUR' ) . ' ';
			
		$this->_db->setQuery( $query );
		$rows = $this->_db->loadAssocList();
		
		// loop through sources
		foreach ($rows as $row_num => $row) {
			// count repositories
			$teller++;
			
			// start every loop with empty source record
			$this->sources->loadEmpty();
			
			// retrieve source_id for source record
			$source_id = $row['value'];
			
			// check whether a record exists with source id
			if ($this->sources->load( $source_id )) {
				// record exists: existing record will be updated
				$ind_update = true;
			} else {
				// record does not exists: new record will be inserted
				$ind_update = false;
				$this->sources->set( 'id', $source_id );
			}
			
			// select all gedcom object lines for this source
			$query = 'SELECT   * '
				.'FROM     #__joaktree_gedcom_objectlines '
				.'WHERE    object_id = ' . $this->_db->Quote( $source_id ) . ' '
				.'ORDER BY order_nr ';
				
			$this->_db->setQuery( $query );
			$row_lines = $this->_db->loadAssocList();
			
			// loop through lines related to the source
			foreach ($row_lines as $row_line_num => $row_line) {
				switch ($row_line['level']) {
					case "1": switch ($row_line['tag']) {
							case "AUTH": 	$this->sources->set( 'author', $row_line['value']);
									BREAK;
							case "TITL":	$this->sources->set( 'title', $row_line['value']);
									BREAK;
							case "PUBL":	$this->sources->set( 'publication', $row_line['value']);
									BREAK;
							case "REPO":	$this->sources->set( 'repo_id', rtrim(ltrim( $row_line['value'], '@'), '@') );
									BREAK;
							default: 	BREAK;
						 }
						 BREAK;
					default: BREAK;
				} // end of level switch
			} // end of loop throuth source lines
			
			if ( $ind_update ) {
				// update existing record
				$ret = $this->sources->_db->updateObject( $this->sources->_tbl, $this->sources, $this->sources->_tbl_key ); //, true );
			} else {
				// insert new record
				$ret = $this->sources->_db->insertObject( $this->sources->_tbl, $this->sources, $this->sources->_tbl_key );
			}
			
			// if insert or update went ok, continue with next source; else stop
			if ( !$ret ) { 
				$msg = 'no succes with processing source ' . $source_id ; 
				return $msg;
			}
		} // end loop through sources
		
		$msg = 'Number of sources processed: ' . $teller;	
		return $msg;
	}


	/*
	** Main function to process all repositories from gedcom file.
	*/
	function process_repos() {
		// initialize table
		$gedcom_lines = & JTable::getInstance('joaktree_gedcom_objectlines', 'Table');
		$this->repos = & JTable::getInstance('joaktree_repositories', 'Table');
		
		// initialize counters
		$teller = 0;
		
		// select all repositories from gedcom table
		$query = 'SELECT * '
			.'FROM #__joaktree_gedcom_objects '
			.'WHERE tag = ' . $this->_db->Quote( 'REPO' ) . ' ';
			
		$this->_db->setQuery( $query );
		$rows = $this->_db->loadAssocList();
		
		// loop through repositories
		foreach ($rows as $row_num => $row) {
			// count repositories
			$teller++;
			
			// start every loop with empty repository record
			$this->repos->loadEmpty();
						
			// retrieve repo_id for repository record
			$repo_id = $row['value'];
			
			// check whether a record exists with repo id
			if ($this->repos->load( $repo_id )) {
				// record exists: existing record will be updated
				$ind_update = true;
			} else {
				// record does not exists: new record will be inserted
				$ind_update = false;
				$this->repos->set( 'id', $repo_id );
			}
			
			// select all gedcom object lines for this repository
			$query = 'SELECT   * '
				.'FROM     #__joaktree_gedcom_objectlines '
				.'WHERE    object_id = ' . $this->_db->Quote( $repo_id ) . ' '
				.'ORDER BY order_nr ';
				
			$this->_db->setQuery( $query );
			$row_lines = $this->_db->loadAssocList();
			
			// loop through lines related to the repository
			foreach ($row_lines as $row_line_num => $row_line) {
				switch ($row_line['level']) {
					case "1": switch ($row_line['tag']) {
							case "NAME":	$this->repos->set( 'name', $row_line['value']);
									BREAK;
							case "WWW":	$this->repos->set( 'website', $row_line['value']);
									BREAK;
							default:	BREAK;
						  }
						  BREAK;
					default:  BREAK;
				} // end of level switch
			} // end of loop throuth repository lines
			
			if ( $ind_update ) {
				// update existing record
				$ret = $this->repos->_db->updateObject( $this->repos->_tbl, $this->repos, $this->repos->_tbl_key, true );
			} else {
				// insert new record
				$ret = $this->repos->_db->insertObject( $this->repos->_tbl, $this->repos, $this->repos->_tbl_key );
			}
			
			// if insert or update went ok, continue with next source; else stop
			if ( !$ret ) { 
				$msg = 'no succes with processing repository ' . $repo_id ; 
				return $msg;
			}
		} // end loop through repositories
		
		$msg = 'Number of repositories processed: ' . $teller;
		return $msg;
	}


	/*
	** Main function to process all persons from gedcom file.
	*/
	function process() {
		// initialize tables
		$gedcom_lines         = & JTable::getInstance('joaktree_gedcom_objectlines', 'Table');
		$this->persons        = & JTable::getInstance('joaktree_persons', 'Table');
		$this->relations      = & JTable::getInstance('joaktree_relations', 'Table');
		$this->person_sources = & JTable::getInstance('joaktree_person_sources', 'Table');
		
		// initialize counters
		$teller = 0;
		
		// select all persons from gedcom table
		$query = 'SELECT * '
			.'FROM #__joaktree_gedcom_objects '
			.'WHERE tag = ' . $this->_db->Quote( 'INDI' ) . ' ';
			
		$this->_db->setQuery( $query );
		$rows = $this->_db->loadAssocList();
		
		// loop through persons
		foreach ($rows as $row_num => $row) {
			// every loop: update counter and start with empty record
			$teller++;
			$this->persons->loadEmpty();
			
			// retrieve preson_id for person record
			$person_id = $row['value'];

			// check whether a record exists with person id
			if ($this->persons->load( $person_id )) {
				// record exists: existing record will be updated
				$ind_update = true;
			} else {
				// record does not exists: new record will be inserted
				$ind_update = false;
				$this->persons->set( 'id', $person_id );
			}
			
			// initialize sources, relations, spouses for this person
			$retdelete = $this->person_sources->deleteSources ($person_id);
			$this->person_sources->set('person_id', $person_id );
			$this->person_sources->set('source_id', null );
			if ($this->trunc_rel) {
				$retdelete = $this->relations->deleteRelations ($person_id);
			}
			$this->number_spouse = 0;
			
			// select all gedcom object lines for this person
			$query = 'SELECT   * '
				.'FROM     #__joaktree_gedcom_objectlines '
				.'WHERE    object_id = ' . $this->_db->Quote( $person_id ) . ' '
				.'ORDER BY order_nr ';
				
			$this->_db->setQuery( $query );
			$row_lines = $this->_db->loadAssocList();
			
			// loop through lines related to the person
			foreach ($row_lines as $row_line_num => $row_line) {
				switch ($row_line['level']) {
					case "1": // set $informantion to be used for level 2 info
						  $information = $row_line['tag'];
						  switch ($information) {
							  case "NAME":	// extract first name and family name
							  		$names  = explode("/", $row_line['value'], 3);
									
									if ($this->patronymSetting == 9) {
										// extract patronym from name field using patronymString-separation
										$tmpName = explode($this->patronymString, $names[0], 3);
										$this->persons->set( 'firstName', $tmpName[0]);
										$this->persons->set( 'patronym', $tmpName[1]);
									} else {
										$this->persons->set( 'firstName', $names[0]);
									}
									
									// set the familyName and namePreposition
									$this->setFamilyName($names[1]); 
									BREAK;
							  case "SEX":	$this->persons->set( 'sex', $row_line['value']);
									BREAK;
							  case "FAMS":	// FAMS is the family this person is a spouse
									// in this family a spouse can be found
									$ret = $this->spouse( $person_id, rtrim(ltrim( $row_line['value'], '@'), '@') );
									BREAK;
							  case "FAMC":	// FAMC is the family this person is a child
									// in this family the parents can be found
									$ret = $this->parents( $person_id, rtrim(ltrim( $row_line['value'], '@'), '@') );
									BREAK;
							  default:	BREAK;
						  }
						  BREAK;
					case "2": switch ($row_line['tag']) {
							  case "DATE":	if ($information == 'BIRT')  { $this->persons->set( 'birthDate', $row_line['value']); }
									if ($information == 'DEAT') { $this->persons->set( 'deathDate', $row_line['value']); }
									if ($information == 'BURI') { $this->persons->set( 'burialDate', $row_line['value']); }
									BREAK;
							  case "PLAC":	if ($information == 'BIRT') { $this->persons->set( 'birthPlace', $row_line['value']); }
									if ($information == 'DEAT') { $this->persons->set( 'deathPlace', $row_line['value']); }
									if ($information == 'BURI') { $this->persons->set( 'burialPlace', $row_line['value']); }
									BREAK;
							  case "SOUR":	// collect all sources for this person
									$ret = $this->setSource( rtrim(ltrim( $row_line['value'], '@'), '@') );
									BREAK;
							  default:	BREAK;
						  }
						  BREAK;
					default:  switch ($row_line['tag']) {
							  case "SOUR":	// collect all sources for this person
									$ret = $this->setSource( rtrim(ltrim( $row_line['value'], '@'), '@') );
									BREAK;
							  case "PAGE":	$this->person_sources->set('sourceText', $row_line['value'] );
									BREAK;
							  default:	BREAK;
						  }
						  BREAK;
				} // end of level switch
			} // end of loop throuth lines related to person
			
			// save the last source information
			if ( $this->person_sources->get('source_id') ) {
				$ret = $this->add_person_source();
			}

			if ( $ind_update ) {
				// update existing record
				$ret = $this->persons->_db->updateObject( $this->persons->_tbl, $this->persons, $this->persons->_tbl_key, true );
			} else {
				// insert new record for admin table
				$ret = $this->admin_person();
				
				// insert record in person table when insert in admin table went ok
				if ($ret) {
					$ret = $this->persons->_db->insertObject( $this->persons->_tbl, $this->persons, $this->persons->_tbl_key );
				}
			}
			
			// if insert or update went ok, continue with next source; else stop
			if ( !$ret ) { 
				$msg = 'no succes with processing person ' . $person_id ; 
				return $msg;
			}
			
		} // end of loop through all persons
		
		// if setting is NOT to retrieve patronyms from name string, patronyms have to determined now
		if ($this->patronymSetting != 9) {
			$ret = $this->getPatronyms();
		}
		
		
		$msg = 'Number of persons processed: ' . $teller;
		return $msg;
	}


	/*
	** Funtion to create an array (table) for conversion ANSEL characters to
	** Unicode characters.
	** Function is taken from JGEN by Solventus (http://solventus.so.funpic.de)
	** The ANSEL to Unicode conversion is based on the table by Heiner Eichmann
	** http://www.heiner-eichmann.de
	*/
	private function prepareANSEL() {
		global $ANSELtable;
		
		// retrieve information name and location of conversion file
		$path     = JPATH_COMPONENT_ADMINISTRATOR.DS.'assets'.DS.'conversion'; 
		$filename = $path.DS.'ans2uni.con';
		
		// check if conversion file exists
		if ( !file_exists( $filename) ) {
			$msg = 'ANSEL-to-UNICODE conversion file not found: (' . $filename . ')';
			return $msg;
		}

		$ANSELfile = file($filename);
		
		// loop through the file and store in array $ANSELtable
		foreach ($ANSELfile as $ANSELline) {
			list($ANSEL,$UNICODEt)  = explode("=",$ANSELline);
			list($hex1,$hex2,$hex3) = explode("+",$ANSEL);
			$hex12=$hex1.$hex2;
			$hex123=$hex12.$hex3;
			
			$UNICODE = substr($UNICODEt,0,4);
			
			if ($hex2 != "") {
				$ANSELtable[$hex1] = "+";
				if ($hex3 != "") {
					$ANSELtable[$hex12] = "+";
					$ANSELtable[$hex123] = $UNICODE;
				} else {
					$ANSELtable[$hex12] = $UNICODE;
				}
			} else {
				$ANSELtable[$hex1] = $UNICODE;
			}
		}
		$msg = 'ANSEL conversion table loaded';
		return $msg;
	}


	/*
	** Funtion to convert a string ANSEL characters to UNICODE or UTF-8
	** Function for conversion from ANSEL to UNICODE is taken from JGEN 
	** by Solventus (http://solventus.so.funpic.de)
	** Function for conversion of UNICODE to UTF-8 by Henri Sivonen, hsivonen@iki.fi,
	** based on Mozilla Communicator client code (http://www.mozilla.org/NPL/)
	*/
	private function convertANSEL($instring) {
		// Initialization
		global $ANSELtable;
		$length = strlen($instring);
		$counter = 0;
		$outstring = "";
		
		// loop through the string
		while ($counter <= $length) {
			// extract next character
			$chr1 = substr($instring,$counter,1);
			
			// convert character to ASCII integer
			$ord1 = ord($chr1);
			
			// convert ASCII integer to hexadecimal string (2 positions)
			$hex1 = strtoupper(dechex($ord1));
			if (strlen($hex1)==1) { $hex1="0".$hex1;}
			
			// character is extracted, so we move one position along the intial string
			$counter++;
			
			// analyse the character
			if ($ord1<128) {
				// for ASCII integer values to 128, no conversion is needed
				$outstring .= $chr1;
			} else {	
				// conversion is needed 
				// check whether the character itself is in conversion table
				if ($ANSELtable[$hex1] == "+") {
					// character is NOT independently in conversion table
					// extract next character, and convert to hexadecimal string (2 positions)
					$chr2 = substr($instring,$counter,1);
					$hex2 = strtoupper(dechex(ord($chr2)));
					if (strlen($hex2)==1) { $hex2="0".$hex2;}
					
					// character is extracted, so we move one position along the intial string
					$counter++;
					
					// check whether the two characters are in conversion table
 					$hex12 = $hex1.$hex2;
					if ($ANSELtable[$hex12] == "+") {
						// two character combination is NOT in conversion table
						// extract next character, and convert to hexadecimal string (2 positions)
						$chr3 = substr($instring,$counter,1);
						$hex3 = strtoupper(dechex(ord($chr3)));
						if (strlen($hex3)==1) { $hex3="0".$hex3;}
						
						// character is extracted, so we move one position along the intial string
						$counter++;
						
						// retrieve three character combination from conversion table
						// and take its UNICODE value.
						$hex123 = $hex12.$hex3;
						$UNICODE = $ANSELtable[$hex123];
						
					} else {
						// two character combination is in conversion table
						// take its UNICODE value
						$UNICODE = $ANSELtable[$hex12];
					}
				} else {
					// character is by itself in conversion table
					// take its UNICODE value
					$UNICODE = $ANSELtable[$hex1];
				}
				
				// ANSEL character is converted to UNICODE
				if ($this->unicode2utf == true) {
					// now conversion from unicode to UTF-8 starts 
					// when so indicated in parameters
					$dest = '';
					$src  = hexdec($UNICODE);

					if(($src >= 0) && ($src <= 0x007f)) {
						$dest .= chr($src);
					} else if ($src <= 0x07ff) {
						$dest .= chr(0xc0 | ($src >> 6));
						$dest .= chr(0x80 | ($src & 0x003f));
					} else if($src == 0xFEFF) {
						// nop -- zap the BOM
						$dest = '';
					} else if ($src >= 0xD800 && $src <= 0xDFFF) {
						// found a surrogate
						$dest = '';
					} else if ($src <= 0xffff) {
						$dest .= chr(0xe0 | ($src >> 12));
						$dest .= chr(0x80 | (($src >> 6) & 0x003f));
						$dest .= chr(0x80 | ($src & 0x003f));
					} else if ($src <= 0x10ffff) {
						$dest .= chr(0xf0 | ($src >> 18));
						$dest .= chr(0x80 | (($src >> 12) & 0x3f));
						$dest .= chr(0x80 | (($src >> 6) & 0x3f));
						$dest .= chr(0x80 | ($src & 0x3f));
					} else { 
						// out of range
						$dest = '';
					}

					$outstring .= $dest;
				} else {
					// no conversion from UNICODE to UTF
					$outstring .= chr($UNICODE); 
				}
			}
		}
		
		return $outstring;
	}


	/*
	** Main function to import the gedcom file.
	*/
	function import_file() {
		// initialize tables and records
		$gedcom_obj   = & JTable::getInstance('joaktree_gedcom_objects', 'Table');
		$gedcom_lines = & JTable::getInstance('joaktree_gedcom_objectlines', 'Table');
		$gedcom_obj->truncate();
		$gedcom_lines->truncate();
		$gedcom_obj->set( 'id' , 0); 
		$gedcom_lines->set( 'id' , 0); 
		$obj_value_id = 0;
		
		// What type of conversion
		$conversion_type = $this->params->get('unicode2utf');
		if ($conversion_type == 0) {
			// no conversion
			$this->conversion = false;
		} else if (($conversion_type == 1)) {
			// conversion from ANSEL to UTF-8
			$this->conversion = true;
			$this->unicode2utf = true;
		} else if (($conversion_type == 2)) {
			// conversion from ANSEL to Unicode
			$this->conversion = true;
			$this->unicode2utf = false;
		} else {
			// parameter has unknown value: no conversion
			$this->conversion = false;
		}
		
		// initialize counters
		$teller0 = 0; // counter for correctly stored gedcom objects
		$teller1 = 0; // counter for errors while storing gedcom objects
		$tellert = 0; // counter for total number of lines in file

		// retrieve information name and location of gedcom file
		$path     = JPATH_ROOT.DS.$this->params->get('gedcomfile_path'); 
		$filename = $path.DS.$this->params->get('gedcomfile_name');
		
		// check if gedcom file exists
		if ( !file_exists( $filename) ) {
			$msg = 'GEDCOM file not found: (' . $filename . ')';
			return $msg;
		}

		// Loop through the array looking for header info.
		$ansel = false;
		$char_done = false;
		$vers_done = false;

		// open file
		$handle = @fopen($filename, "r");
		if ($handle) {
			// loop through the lines
			while (!feof($handle)) {
				$line = fgets($handle, 4096);
				// we are ready
				if ( ($char_done) and ($vers_done) ) {
					BREAK;
				}
				
				// remove end-of-line characters
				$line      = rtrim($line, "\r\n");
				
				// split line into three parts with space as deviding character
				$elements  = explode(" ", $line, 3);
				
				// first part of line is the level; 
				$level     = $elements[0]; 
				
				// process only the header: so is this the header
				if ($level == 0) {
					if ($elements[1] == 'HEAD') {
						$ind_header = true;
					} else {
						$ind_header = false;
					}
				}
				
				// process only the header
				if ($ind_header == true) {
					// see whether we have to transer ANSEL to UTF-8
					// other character sets are left alone.
					if ($elements[1] == 'CHAR') {
						$char_done = true;
						if ($elements[2] == 'ANSEL') {
							$ansel = true;
						}
					}
					
					// check the version of GEDCOM: is this the GEDCOM and not SOURCE?
					if ($elements[1] == 'GEDC') {
						$ind_get_vers = true;
					}
					
					// check version of GEDCOM
					if ( ($elements[1] == 'VERS') and ($ind_get_vers) ) { 
						$vers_done = true;
						$version = $elements[2];
						if ($version != '5.5') {
							$msg = 'Version of GEDCOM file is not v5.5';
							return $msg;
						}
					}
				} // end of if ind_header == true
			} // end of loop
		}
		fclose($handle);
		
		// if charcter set is ANSEL
		if ( ($this->conversion == true) and ($ansel == true) ) {
			$ret = $this->prepareANSEL();
			$msg = $msg . '+' . $ret;
		}
		
		// Loop through the array.
		// open file
		$handle = @fopen($filename, "r");
		if ($handle) {
			// loop through the lines
			while (!feof($handle)) {
				$line = fgets($handle, 4096);
				
				$tellert++;
				
				// remove end-of-line characters
				$line      = rtrim($line, "\r\n");
				
				// if ANSEL convert the line
				if ( ($this->conversion == true) and ($ansel == true) ) {
					$line = $this->convertANSEL( $line );
				}
				
				// replace special characters
				// & (= ampersand)
				$line = str_replace("&", "&#38;", $line);
				// < (= less than sign)
				$line = str_replace("<", "&#60;", $line);
				// > (= greater than sign)
				$line = str_replace(">", "&#62;", $line);
				
				// split line into three parts with space as deviding character
				$elements  = explode(" ", $line, 3);
				
				// first part of line is the level; 
				$level     = $elements[0]; 
	
				// the level 0 lines are the main lines (=gedcom objects)
				// these lines are stored in object table
				// when level is greater than 0, it is a depending line and 
				// stored in object-line table
				if ($level == 0) {
					// level = 0: we dealing with the object
					if ($elements[2] == null) {
						// third element is empty for level 0 line
						// tag is in the second element
						// there is no value. this will be filled with value from counter object_value_id
						$obj_value_id++;
						$obj_value = $obj_value_id;
						$gedcom_obj->set( 'tag',   $elements[1] ); 
						$gedcom_obj->set( 'value' , $obj_value); 
					} else {
						// third element is not empty. this countains the tag
						// second element contains the value
						// value has to be stripped from @ characters
						$obj_value = ltrim(rtrim($elements[1], '@'), '@');
						$gedcom_obj->set( 'tag',   $elements[2] ); 
						$gedcom_obj->set( 'value', $obj_value ); 
					}
					
					// after retrieving the tag and value information record can be stored.
					// counters are updated
					if ($gedcom_obj->store() ) {
						$teller0++;
					} else {
						$teller1++;
					}
					
					// necessary line for submit (is this line really needed? )
					$gedcom_obj->set( 'id' , $gedcom_obj->insertid); 
					
					// initialize the teller for the related object lines for this object
					$order_nr = 0;
	 
				} else {
					// level <> 0: we dealing with object lines
					
					// update counter for object lines
					$order_nr++;
					
					// element 2 contains the tag; element 3 contains the value
					// set all attributes for the object line
					$gedcom_lines->set( 'id' , $gedcom_lines->insertid); 
					$gedcom_lines->set( 'object_id' , $obj_value); 
					$gedcom_lines->set( 'order_nr' , $order_nr); 
					$gedcom_lines->set( 'level' , $level); 
					$gedcom_lines->set( 'tag' , $elements[1] ); 
					$gedcom_lines->set( 'value' , $elements[2] ); 
	
					// store the object line
					$gedcom_lines->store();
	
					// necessary line for submit (is this line really needed? )
					$gedcom_lines->set( 'id' , $gedcom_lines->insertid); 
					
				} // end of check on level = 0
			} // end of loop through array
		}
		fclose($handle);

		$msg = $msg . '+' . 'Import is finished';
		return $msg;
    }
}
?>